from DisplayTarget import DisplayTarget
import gtk


#
# Abstract class for targets that can contain others.
#
class ContainerTarget(DisplayTarget):


    def __init__(self, parent, display):

        # the list is necessary for preserving order
        self.__children = []

        # mapping: id -> child
        self.__children_id = {}

        # mapping: child -> id
        self.__ids = {}
        
        self.__settings = []
        

        DisplayTarget.__init__(self, parent, display)



    def _register_child(self, child, cid):

        self.__children_id[cid] = child
        self.__ids[child] = cid
        self.__children.append(child)



    def _unregister_child(self, child):

        cid = self.__ids[child]
        if (self.__children_id.has_key(cid)): del self.__children_id[cid]
        if (self.__ids.has_key(child)): del self.__ids[child]
        self.__children.remove(child)



    def add_children(self, childrendata):

        for childname, childclass, childsettings, children in childrendata:
            self.add_child(childname, childclass, childsettings, children)



    def add_child(self, cid, clss, settings, children):

        display = self._get_display()
        child = clss(self, display)
        if (children): child.add_children(children)

        self._register_child(child, cid)

        if (settings):
            for key, value in settings.get_entries():
                self.__settings.append((child, key, value))



    def _init_children(self):

        for child, key, value in self.__settings:
            child.set_config(key, value)
        

        

    def _get_children(self): return self.__children[:]

    def _get_child(self):

        children = self._get_children()
        if (not children): return None
        else: return children[0]

    def get_child_by_id(self, id): return self.__children_id.get(id)

    def get_id_by_child(self, child): return self.__ids.get(child)



    #
    # Asks the children with percentual sizes to collapse, so the container
    # may shrink.
    #
    def collapse_siblings(self, child, col_x, col_y):

        # check if the container itself has to collapse
        is_px, is_py, is_pw, is_ph = self.get_percentual_state()
        if ((is_px and col_x) or (is_py and col_y) or
            (is_pw and col_x) or (is_ph and col_y)):
            self.hide()
            gtk.idle_add(self.show)
            return
        
        for c in self._get_children():
            if (c == child): continue
            
            is_px, is_py, is_pw, is_ph = c.get_percentual_state()
            if (((is_px and col_x) or (is_py and col_y) or
                (col_x and is_pw) or (col_y and is_ph)) and
                c.get_property("visible")):
                
                c.hide()
                gtk.idle_add(c.show)
            #end if
        #end for


    def delete(self):

        DisplayTarget.delete(self)
        for c in self._get_children():
            c.delete()
            self._unregister_child(c)
        self.__children = []
        self.__children_id.clear()
        self.__ids.clear()
